@@ -64,36 +64,36 @@ static void
 atk_text_base_init (gpointer *g_class)
 {
   static gboolean initialized = FALSE;
-
+  
   if (! initialized)
     {
-   /* 
-    * Note that text_changed signal supports details "insert", "delete", 
-    * possibly "replace". 
-    */
-
-    atk_text_signals[TEXT_CHANGED] =
-      g_signal_newc ("text_changed",
-                     ATK_TYPE_TEXT,
-                     G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                     G_STRUCT_OFFSET (AtkTextIface, text_changed), 
-                     (GSignalAccumulator) NULL, NULL,
-                     atk_marshal_VOID__INT_INT,
-                     G_TYPE_NONE,
-                     2, G_TYPE_INT, G_TYPE_INT);
-
-    atk_text_signals[CARET_MOVED] =
-      g_signal_newc ("text_caret_moved",
-                     ATK_TYPE_TEXT,
-                     G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET (AtkTextIface, caret_changed),
-                     (GSignalAccumulator) NULL, NULL,
-                     g_cclosure_marshal_VOID__INT,
-                     G_TYPE_NONE,
-                     1, G_TYPE_INT);
-
-  initialized = TRUE;
-  }
+      /* 
+       * Note that text_changed signal supports details "insert", "delete", 
+       * possibly "replace". 
+       */
+      
+      atk_text_signals[TEXT_CHANGED] =
+	g_signal_new ("text_changed",
+		      ATK_TYPE_TEXT,
+		      G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+		      G_STRUCT_OFFSET (AtkTextIface, text_changed), 
+		      (GSignalAccumulator) NULL, NULL,
+		      atk_marshal_VOID__INT_INT,
+		      G_TYPE_NONE,
+		      2, G_TYPE_INT, G_TYPE_INT);
+      
+      atk_text_signals[CARET_MOVED] =
+	g_signal_new ("text_caret_moved",
+		      ATK_TYPE_TEXT,
+		      G_SIGNAL_RUN_LAST,
+		      G_STRUCT_OFFSET (AtkTextIface, caret_changed),
+		      (GSignalAccumulator) NULL, NULL,
+		      g_cclosure_marshal_VOID__INT,
+		      G_TYPE_NONE,
+		      1, G_TYPE_INT);
+      
+      initialized = TRUE;
+    }
 }
 
 /**
@@ -112,8 +112,7 @@ atk_text_get_text (AtkText      *text,
                    gint         end_offset)
 {
   AtkTextIface *iface;
-
-  g_return_val_if_fail (text != NULL, NULL);
+  
   g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -125,7 +124,7 @@ atk_text_get_text (AtkText      *text,
 }
 
 /**
- * atk_text_get_character_at_offset
+ * atk_text_get_character_at_offset:
  * @text: an #AtkText
  * @offset: position
  *
@@ -139,7 +138,6 @@ atk_text_get_character_at_offset (AtkText      *text,
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, (gunichar) 0);
   g_return_val_if_fail (ATK_IS_TEXT (text), (gunichar) 0);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -151,10 +149,12 @@ atk_text_get_character_at_offset (AtkText      *text,
 }
 
 /**
- * atk_text_get_text_after_offset
+ * atk_text_get_text_after_offset:
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
+ * @start_offset: the start offset of the returned string.
+ * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
  * If the boundary type is ATK_TEXT_BOUNDARY_WORD_START or
@@ -170,26 +170,40 @@ atk_text_get_character_at_offset (AtkText      *text,
 gchar*
 atk_text_get_text_after_offset (AtkText          *text,
                                 gint             offset,
-                                AtkTextBoundary  boundary_type)
+                                AtkTextBoundary  boundary_type,
+ 				gint             *start_offset,
+				gint		 *end_offset)
 {
   AtkTextIface *iface;
+  gint local_start_offset, local_end_offset;
+  gint *real_start_offset, *real_end_offset;
 
-  g_return_val_if_fail (text != NULL, NULL);
   g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
 
+  if (start_offset)
+    real_start_offset = start_offset;
+  else
+    real_start_offset = &local_start_offset;
+  if (end_offset)
+    real_end_offset = end_offset;
+  else
+    real_end_offset = &local_end_offset;
+
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_text_after_offset)
-    return (*(iface->get_text_after_offset)) (text, offset, boundary_type);
+    return (*(iface->get_text_after_offset)) (text, offset, boundary_type, real_start_offset, real_end_offset);
   else
     return NULL;
 }
 
 /**
- * atk_text_get_text_at_offset
+ * atk_text_get_text_at_offset:
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
+ * @start_offset: the start offset of the returned string.
+ * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
  * If the boundary_type is ATK_TEXT_BOUNDARY_WORD_START or 
@@ -204,26 +218,40 @@ atk_text_get_text_after_offset (AtkText          *text,
 gchar*
 atk_text_get_text_at_offset (AtkText          *text,
                              gint             offset,
-                             AtkTextBoundary  boundary_type)
+                             AtkTextBoundary  boundary_type,
+			     gint             *start_offset,
+			     gint             *end_offset)
 {
   AtkTextIface *iface;
+  gint local_start_offset, local_end_offset;
+  gint *real_start_offset, *real_end_offset;
 
-  g_return_val_if_fail (text != NULL, NULL);
   g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
 
+  if (start_offset)
+    real_start_offset = start_offset;
+  else
+    real_start_offset = &local_start_offset;
+  if (end_offset)
+    real_end_offset = end_offset;
+  else
+    real_end_offset = &local_end_offset;
+
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_text_at_offset)
-    return (*(iface->get_text_at_offset)) (text, offset, boundary_type);
+    return (*(iface->get_text_at_offset)) (text, offset, boundary_type, real_start_offset, real_end_offset);
   else
     return NULL;
 }
 
 /**
- * atk_text_get_text_before_offset
+ * atk_text_get_text_before_offset:
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
+ * @start_offset: the start offset of the returned string.
+ * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
  * If the boundary type is ATK_TEXT_BOUNDARY_WORD_START or
@@ -239,36 +267,46 @@ atk_text_get_text_at_offset (AtkText          *text,
 gchar*
 atk_text_get_text_before_offset (AtkText          *text,
                                  gint             offset,
-                                 AtkTextBoundary  boundary_type)
+                                 AtkTextBoundary  boundary_type,
+				 gint             *start_offset,
+				 gint		  *end_offset)
 {
   AtkTextIface *iface;
+  gint local_start_offset, local_end_offset;
+  gint *real_start_offset, *real_end_offset;
 
-  g_return_val_if_fail (text != NULL, NULL);
   g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
 
+  if (start_offset)
+    real_start_offset = start_offset;
+  else
+    real_start_offset = &local_start_offset;
+  if (end_offset)
+    real_end_offset = end_offset;
+  else
+    real_end_offset = &local_end_offset;
+
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_text_before_offset)
-    return (*(iface->get_text_before_offset)) (text, offset, boundary_type);
+    return (*(iface->get_text_before_offset)) (text, offset, boundary_type, real_start_offset, real_end_offset);
   else
     return NULL;
 }
 
 /**
- * atk_text_get_caret_offset
+ * atk_text_get_caret_offset:
  * @text: an #AtkText
  *
  * Gets the offset position of the caret (cursor).
  *
- * Returns: the offset position of the caret (cursor), or -1 if
- *    one does not exist (or is not supported by the widget).
+ * Returns: the offset position of the caret (cursor).
  **/
 gint
 atk_text_get_caret_offset (AtkText *text)
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, -1);
   g_return_val_if_fail (ATK_IS_TEXT (text), -1);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -285,11 +323,11 @@ atk_text_get_caret_offset (AtkText *text)
  * @offset: position
  * @x: x-position of character
  * @y: y-position of character
- * @length: length of character
  * @width: width of character
+ * @height: height of character
  * @coords: specify whether coordinates are relative to the screen or widget window 
  *
- * Given an @offset, the @x, @y, @length, and @width values are filled
+ * Given an @offset, the @x, @y, @width, and @height values are filled
  * appropriately. 
  **/
 void
@@ -297,25 +335,43 @@ atk_text_get_character_extents (AtkText *text,
                                 gint offset,
                                 gint *x,
                                 gint *y,
-                                gint *length,
                                 gint *width,
-			        AtkXYCoords coords)
+                                gint *height,
+			        AtkCoordType coords)
 {
   AtkTextIface *iface;
+  gint local_x, local_y, local_width, local_height;
+  gint *real_x, *real_y, *real_width, *real_height;
 
-  g_return_if_fail (text != NULL);
   g_return_if_fail (ATK_IS_TEXT (text));
 
+  if (x)
+    real_x = x;
+  else
+    real_x = &local_x;
+  if (y)
+    real_y = y;
+  else
+    real_y = &local_y;
+  if (width)
+    real_width = width;
+  else
+    real_width = &local_width;
+  if (height)
+    real_height = height;
+  else
+    real_height = local_height;
+
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_character_extents)
-    (*(iface->get_character_extents)) (text, offset, x, y, length, width, coords);
+    (*(iface->get_character_extents)) (text, offset, real_x, real_y, real_width, real_height, coords);
   else
     {
-      *x = 0;
-      *x = 0;
-      *length = 0;
-      *width = 0;
+      *real_x = 0;
+      *real_y = 0;
+      *real_width = 0;
+      *real_height = 0;
     }
 }
 
@@ -329,7 +385,9 @@ atk_text_get_character_extents (AtkText *text,
  *Creates an #AtkAttributeSet which consists of the attributes explicitly
  *set at the position @offset in the text. @start_offset and @end_offset are
  *set to the start and end of the range around @offset where the attributes are
- *invariant.
+ *invariant. See the ATK_ATTRIBUTE macros, such as #ATK_ATTRIBUTE_LEFT_MARGIN
+ *for types of text attributes that can be returned. Note that other 
+ *attributes that do not have corresponding macros may also be returned.
  *
  *Returns: an #AtkAttributeSet which contains the attributes explicitly set
  *at @offset
@@ -340,20 +398,30 @@ AtkAttributeSet* atk_text_ref_run_attributes              (AtkText          *tex
                                                            gint             *end_offset)
 {
   AtkTextIface *iface;
+  gint local_start_offset, local_end_offset;
+  gint *real_start_offset, *real_end_offset;
 
-  g_return_val_if_fail (text != NULL, NULL);
   g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
 
+  if (start_offset)
+    real_start_offset = start_offset;
+  else
+    real_start_offset = &local_start_offset;
+  if (end_offset)
+    real_end_offset = end_offset;
+  else
+    real_start_offset = &local_end_offset;
+
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->ref_run_attributes)
-    return (*(iface->ref_run_attributes)) (text, offset, start_offset, end_offset);
+    return (*(iface->ref_run_attributes)) (text, offset, real_start_offset, real_end_offset);
   else
     return NULL;
 }
 
 /**
- * atk_text_get_character_count
+ * atk_text_get_character_count:
  * @text: an #AtkText
  *
  * Gets the character count.
@@ -365,7 +433,6 @@ atk_text_get_character_count (AtkText *text)
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, -1);
   g_return_val_if_fail (ATK_IS_TEXT (text), -1);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -381,11 +448,12 @@ atk_text_get_character_count (AtkText *text)
  * @text: an #AtkText
  * @x: screen x-position of character
  * @y: screen y-position of character
- * @coords: specify whether coordinates are relative to the screen or widget window 
+ * @coords: specify whether coordinates are relative to the screen or
+ * widget window 
  *
- * Gets the offset of the character located at coordinates @x and @y. @x and @y are
- * interpreted as being relative to the screen or this widget's window depending
- * on @coords.
+ * Gets the offset of the character located at coordinates @x and @y. @x and @y
+ * are interpreted as being relative to the screen or this widget's window
+ * depending on @coords.
  *
  * Returns: the offset to the character which is located at
  * the specified @x and @y coordinates.
@@ -394,11 +462,10 @@ gint
 atk_text_get_offset_at_point (AtkText *text,
                               gint x,
                               gint y,
-			      AtkXYCoords coords)
+			      AtkCoordType coords)
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, -1);
   g_return_val_if_fail (ATK_IS_TEXT (text), -1);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -410,7 +477,7 @@ atk_text_get_offset_at_point (AtkText *text,
 }
 
 /**
- * atk_text_get_n_selections
+ * atk_text_get_n_selections:
  * @text: an #AtkText
  *
  * Gets the number of selected regions.
@@ -423,7 +490,6 @@ atk_text_get_n_selections (AtkText *text)
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, -1);
   g_return_val_if_fail (ATK_IS_TEXT (text), -1);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -435,10 +501,10 @@ atk_text_get_n_selections (AtkText *text)
 }
 
 /**
- * atk_text_get_selection
+ * atk_text_get_selection:
  * @text: an #AtkText
  * @selection_num: The selection number.  The selected regions are
- * assigned numbers that corrispond to how far the region is from the
+ * assigned numbers that correspond to how far the region is from the
  * start of the text.  The selected region closest to the beginning
  * of the text region is assigned the number 0, etc.  Note that adding,
  * moving or deleting a selected region can change the numbering.
@@ -450,27 +516,39 @@ atk_text_get_n_selections (AtkText *text)
  * Returns: the selected text.
  **/
 gchar*
-atk_text_get_selection (AtkText *text, gint selection_num,
-   gint *start_offset, gint *end_offset)
+atk_text_get_selection (AtkText *text, 
+                        gint    selection_num,
+                        gint    *start_offset,
+                        gint    *end_offset)
 {
   AtkTextIface *iface;
+  gint local_start_offset, local_end_offset;
+  gint *real_start_offset, *real_end_offset;
 
-  g_return_val_if_fail (text != NULL, NULL);
   g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
 
+  if (start_offset)
+    real_start_offset = start_offset;
+  else
+    real_start_offset = &local_start_offset;
+  if (end_offset)
+    real_end_offset = end_offset;
+  else
+    real_start_offset = &local_end_offset;
+
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_selection)
   {
     return (*(iface->get_selection)) (text, selection_num,
-       start_offset, end_offset);
+       real_start_offset, real_end_offset);
   }
   else
     return NULL;
 }
 
 /**
- * atk_text_add_selection
+ * atk_text_add_selection:
  * @text: an #AtkText
  * @start_offset: the start position of the selected region
  * @end_offset: the end position of the selected region
@@ -480,12 +558,12 @@ atk_text_get_selection (AtkText *text, gint selection_num,
  * Returns: %TRUE if success, %FALSE otherwise
  **/
 gboolean
-atk_text_add_selection (AtkText *text, gint start_offset,
-   gint end_offset)
+atk_text_add_selection (AtkText *text, 
+                        gint    start_offset,
+                        gint    end_offset)
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, FALSE);
   g_return_val_if_fail (ATK_IS_TEXT (text), FALSE);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -497,24 +575,24 @@ atk_text_add_selection (AtkText *text, gint start_offset,
 }
 
 /**
- * atk_text_remove_selection
+ * atk_text_remove_selection:
  * @text: an #AtkText
  * @selection_num: The selection number.  The selected regions are
- * assigned numbers that corrispond to how far the region is from the
+ * assigned numbers that correspond to how far the region is from the
  * start of the text.  The selected region closest to the beginning
  * of the text region is assigned the number 0, etc.  Note that adding,
  * moving or deleting a selected region can change the numbering.
  *
- * Removes the specified selection
+ * Removes the specified selection.
  *
  * Returns: %TRUE if success, %FALSE otherwise
  **/
 gboolean
-atk_text_remove_selection (AtkText *text, gint selection_num)
+atk_text_remove_selection (AtkText *text, 
+                           gint    selection_num)
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, FALSE);
   g_return_val_if_fail (ATK_IS_TEXT (text), FALSE);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -526,27 +604,28 @@ atk_text_remove_selection (AtkText *text, gint selection_num)
 }
 
 /**
- * atk_text_set_selection
+ * atk_text_set_selection:
  * @text: an #AtkText
  * @selection_num: The selection number.  The selected regions are
- * assigned numbers that corrispond to how far the region is from the
+ * assigned numbers that correspond to how far the region is from the
  * start of the text.  The selected region closest to the beginning
  * of the text region is assigned the number 0, etc.  Note that adding,
  * moving or deleting a selected region can change the numbering.
  * @start_offset: the new start position of the selection
  * @end_offset: the new end position of the selection
  *
- * Changes the start and end offset of the specified selection
+ * Changes the start and end offset of the specified selection.
  *
  * Returns: %TRUE if success, %FALSE otherwise
  **/
 gboolean
-atk_text_set_selection (AtkText *text, gint selection_num,
-   gint start_offset, gint end_offset)
+atk_text_set_selection (AtkText *text, 
+                        gint    selection_num,
+                        gint    start_offset, 
+                        gint    end_offset)
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, FALSE);
   g_return_val_if_fail (ATK_IS_TEXT (text), FALSE);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -561,7 +640,7 @@ atk_text_set_selection (AtkText *text, gint selection_num,
 }
 
 /**
- * atk_text_set_caret_offset
+ * atk_text_set_caret_offset:
  * @text: an #AtkText
  * @offset: position
  *
@@ -575,7 +654,6 @@ atk_text_set_caret_offset (AtkText *text,
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (text != NULL, FALSE);
   g_return_val_if_fail (ATK_IS_TEXT (text), FALSE);
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -589,3 +667,26 @@ atk_text_set_caret_offset (AtkText *text,
       return FALSE;
     }
 }
+
+/**
+ * atk_attribute_set_free:
+ * @attrib_set: The #AtkAttributeSet to free
+ *
+ * Frees the memory used by an #AtkAttributeSet, including all its
+ * #AtkAttributes.
+ **/
+void
+atk_attribute_set_free(AtkAttributeSet *attrib_set)
+{
+  gint index;
+
+  if (attrib_set == NULL)
+    return;
+
+  for (index = 0; index < g_slist_length(attrib_set); index++)
+  {
+    g_free(((AtkAttribute*) (g_slist_nth(attrib_set,index)->data))->name);
+    g_free(((AtkAttribute*) (g_slist_nth(attrib_set,index)->data))->value);
+  }
+  g_slist_free(attrib_set);
+}
